#!/bin/sh

# System install script. This script takes care of all configuration and
# installation tasks to provide a working iGp IOC/display system. Requires
# a Fedora Core 8 installation. Must be run as a root
#
# Copyright (c) 2006-2007 Dimtel, Inc., All Rights Reserved
#
# $Id: install.sh,v 1.6 2009/02/27 17:52:41 dim Exp $
# $Date: 2009/02/27 17:52:41 $
# $Author: dim $
# $Revision: 1.6 $
# $Log: install.sh,v $
# Revision 1.6  2009/02/27 17:52:41  dim
# Updated install script to handle multiple distributions
#
# Revision 1.5  2007/12/30 15:29:01  dim
# Script updates for Fedora 8 client software, configuration for IOC
# installations on different motherboards (boot device, network driver)
#
# Revision 1.4  2007/03/14 08:36:13  dim
# Extensive installation script updates
#
# Revision 1.3  2006/12/28 09:57:55  dim
# Installation cleanup, release updates
#
# Revision 1.2  2006/12/11 09:29:15  dim
# Installation process cleanup, EDM config updates
#
# Revision 1.1  2006/12/07 23:10:51  dim
# Distribution packaging and installation work
#
#

# Service functions
display_ok()
{
  /usr/bin/tput hpa 60
  echo -n [
  /usr/bin/tput setf 2
  echo -n OK
  /usr/bin/tput setf 0
  echo ]
  return 0
}

display_error()
{
  /usr/bin/tput hpa 60
  echo -n [
  /usr/bin/tput setf 4
  echo -n ERROR
  /usr/bin/tput setf 0
  echo ]
  return 0
}

strstr() {
  [ "${1#*$2*}" = "$1" ] && return 1
  return 0
}

strnstr() {
  [ "${1#*$2*}" = "$1" ] && return 0
  return 1
}

export PATH=/bin:/usr/bin:/sbin:/usr/sbin

# Verify that we are root
if [ $EUID -ne 0 ]; then
  echo "This script must be executed with root permissions"
  exit 1;
fi


# Start from figuring out where we are
dir=`dirname "$0"`
src=`(cd "${dir}"; pwd)`

# Figure out the destination
read -p "Username to install under [iGp]: " user
if [ a${user} == a ]; then user=iGp; fi

read -p "Install directory, relative to ~${user} [iGp]: " dest_dir
if [ a${dest_dir} == a ]; then dest_dir=iGp; fi

echo Source directory: ${src}
echo Install username: ${user}
echo Install directory: ${dest_dir}

# Create the user, if not there yet
echo -n Creating user ${user}:
/usr/sbin/useradd -m ${user} 2> /dev/null
case $? in
  0) 
    display_ok
    /usr/bin/passwd ${user}
    ;;
  9)
     echo -n " installing under existing account:"
     display_ok
     ;;
  *) 
     display_error
     exit 2
     ;;
esac

user_home=`/bin/bash -c "echo ~${user}"`
dest=${user_home}/${dest_dir}

if [ -a ${dest} ]; then
  read -p "Directory/file ${dest} already exists, OK to delete: [y/N]: " answer
  if strstr $"yY" "$answer"; then
    /bin/rm -rf ${dest}
  else
    echo "Installation aborted"
    exit 3
  fi
fi

# Logfile holding installation information
logfile=${dest}/install.log

# Move the distribution to the install point
echo -n "Installing software under ${dest}:"
/usr/bin/rsync -rtl --delete "${src}/iGp/" ${dest}/
# Change ownership
/usr/bin/find ${dest} -print0 | xargs -0 chown ${user}:${user}
# Make all files writable by the user
/usr/bin/find ${dest} -print0 | xargs -0 chmod u+w
# Copy RELEASE over
/bin/cp "${src}/RELEASE" ${dest}/
display_ok

# Install the necessary RPMs
echo -n "Installing necessary RPMs:"
# First of all, disable GPG signature check
/bin/grep gpgcheck=1 /etc/yum.conf > /dev/null
if [ $? -eq 0 ]; then
  tmp=`/bin/mktemp -p /tmp`
  /bin/sed -e 's/gpgcheck=1/gpgcheck=0/' /etc/yum.conf > ${tmp}
  /bin/mv /etc/yum.conf /etc/yum.conf.iGp_save
  /bin/mv ${tmp} /etc/yum.conf
fi
RPMS="libXp xorg-x11-fonts-75dpi xorg-x11-fonts-ISO8859-1-75dpi openmotif22"
/usr/bin/yum install $RPMS -y > /dev/null 2>&1
err=""
for name in ${RPMS}; do
  rpm -q ${name} > /dev/null
  if [ $? -eq 1 ]; then
    err="${name} ${err}"
  fi
done
if [ -n "$err" ]; then
  display_error
  echo Missing: $err
else
  display_ok
fi

# Rebuild edm object files
echo -n "Rebuilding EDM configuration files:"
(cd ${dest}; . ./envSet.sh; edm_config) >> ${logfile} 2>&1
display_ok

# Add environment setup to .bashrc
# Remove old line, if any
dotfile=${user_home}/.bashrc
echo -n "Adding environment setup to ${dotfile}:"
tmp=`/bin/mktemp -p /tmp`
/bin/grep -v "/envSet.sh # Added by iGp install, do not modify" ${dotfile} > ${tmp}
/bin/cp ${dotfile} ${dotfile}.iGp.save
/bin/cp ${tmp} ${dotfile}
/bin/rm ${tmp}
echo ". \${HOME}/${dest_dir}/envSet.sh # Added by iGp install, do not modify" >> ${dotfile}
display_ok

# We are done!
echo "The installation is complete."
cat <<EOF
Do not forget to run "IOC_add <IP address> <device name>" for each IOC. 
For example, for IOC at 192.168.0.111 with device name FB (EPICS
channels start with IGPF:FB:) type: IOC_add 192.168.0.111 FB
EOF
